@@ -20,7 +20,7 @@ When working with grid case results, the following two arguments are used in man
 result
 
 **Result Definition enums**::
-    
+
     property_type           |       | porosity_model
     (str enum)              |       | (str enum)
     ----------------------- | ----- | --------------
@@ -35,8 +35,9 @@ result
 
 """
 
-import builtins
 import grpc
+import uuid
+from typing import List, Tuple
 
 import Case_pb2
 import Case_pb2_grpc
@@ -48,16 +49,17 @@ import Properties_pb2_grpc
 import NNCProperties_pb2
 import NNCProperties_pb2_grpc
 from .resinsight_classes import (
-    Case,
-    EclipseCase,
-    GeoMechCase,
-    WellBoreStabilityPlot,
-    WbsParameters,
+    Case as Case,
+    EclipseCase as EclipseCase,
+    GeoMechCase as GeoMechCase,
+    WellBoreStabilityPlot as WellBoreStabilityPlot,
+    WbsParameters as WbsParameters,
 )
 
-from .grid import Grid
+from .grid import Grid as Grid
+from .project import Project as Project
 from .pdmobject import add_method
-from .view import View
+from .view import View as View
 from .simulation_well import SimulationWell
 import rips.project  # full name import due to circular dependency
 
@@ -74,7 +76,7 @@ def __custom_init__(self, pb2_object, channel):
 
 
 @add_method(Case)
-def __grid_count(self):
+def __grid_count(self) -> int:
     """Get number of grids in the case"""
     try:
         return self.__case_stub.GetGridCount(self.__request()).count
@@ -125,7 +127,7 @@ def __generate_property_input_chunks(self, array, parameters):
 
 
 @add_method(Case)
-def grid(self, index=0):
+def grid(self, index: int = 0) -> Grid:
     """Get Grid of a given index
 
     Arguments:
@@ -138,7 +140,7 @@ def grid(self, index=0):
 
 
 @add_method(Case)
-def grids(self):
+def grids(self) -> List[Grid]:
     """Get a list of all rips Grid objects in the case
 
     Returns:
@@ -166,7 +168,7 @@ def replace(self, new_grid_file):
 
 
 @add_method(Case)
-def cell_count(self, porosity_model="MATRIX_MODEL"):
+def cell_count(self, porosity_model: str = "MATRIX_MODEL") -> int:
     """Get a cell count object containing number of active cells and total number of cells
 
     Arguments:
@@ -193,7 +195,7 @@ def cell_count(self, porosity_model="MATRIX_MODEL"):
 
 
 @add_method(Case)
-def cell_info_for_active_cells_async(self, porosity_model="MATRIX_MODEL"):
+def cell_info_for_active_cells_async(self, porosity_model: str = "MATRIX_MODEL"):
     """Get Stream of cell info objects for current case
 
     Arguments:
@@ -213,7 +215,7 @@ def cell_info_for_active_cells_async(self, porosity_model="MATRIX_MODEL"):
 
 
 @add_method(Case)
-def cell_info_for_active_cells(self, porosity_model="MATRIX_MODEL"):
+def cell_info_for_active_cells(self, porosity_model: str = "MATRIX_MODEL"):
     """Get list of cell info objects for current case
 
     Arguments:
@@ -298,7 +300,7 @@ def reservoir_boundingbox(self):
 
 
 @add_method(Case)
-def reservoir_depth_range(self):
+def reservoir_depth_range(self) -> Tuple[float, float]:
     """Get the reservoir depth range
 
     Returns:
@@ -324,13 +326,30 @@ def view(self, view_id):
     Returns:
         :class:`rips.generated.generated_classes.View`
     """
-    views = self.views()
+    project = self.ancestor(rips.project.Project)
+    views = project.views()
     for view_object in views:
         if view_object.id == view_id:
             return view_object
     return None
 
 
+@add_method(Case)
+def views(self):
+    """Get all views of a case
+
+    Returns:
+        List of :class:`rips.generated.generated_classes.View`
+    """
+    project = self.ancestor(rips.project.Project)
+    views = project.views()
+    views_for_case = []
+    for view_object in views:
+        if view_object.id == self.id:
+            views_for_case.append(view_object)
+    return views_for_case
+
+
 @add_method(Case)
 def create_view(self):
     """Create a new view in the current case
@@ -487,7 +506,7 @@ def create_multiple_fractures(
     if isinstance(well_path_names, str):
         well_path_names = [well_path_names]
     return self._execute_command(
-        createMultipleFractures=Cmd.MultipleFracRequest(
+        createMultipleFractures=Cmd.CreateMultipleFracRequest(
             caseId=self.id,
             templateId=template_id,
             wellPathNames=well_path_names,
@@ -604,7 +623,7 @@ def export_flow_characteristics(
 
 
 @add_method(Case)
-def available_properties(self, property_type, porosity_model="MATRIX_MODEL"):
+def available_properties(self, property_type, porosity_model: str = "MATRIX_MODEL"):
     """Get a list of available properties
 
     For argument details, see :ref:`Result Definition <result-definition-label>`
@@ -626,7 +645,7 @@ def available_properties(self, property_type, porosity_model="MATRIX_MODEL"):
 
 @add_method(Case)
 def active_cell_property_async(
-    self, property_type, property_name, time_step, porosity_model="MATRIX_MODEL"
+    self, property_type, property_name, time_step, porosity_model: str = "MATRIX_MODEL"
 ):
     """Get a cell property for all active cells. Async, so returns an iterator. For argument details, see :ref:`Result Definition <result-definition-label>`
 
@@ -655,7 +674,7 @@ def active_cell_property_async(
 
 @add_method(Case)
 def active_cell_property(
-    self, property_type, property_name, time_step, porosity_model="MATRIX_MODEL"
+    self, property_type, property_name, time_step, porosity_model: str = "MATRIX_MODEL"
 ):
     """Get a cell property for all active cells. Sync, so returns a list. For argument details, see :ref:`Result Definition <result-definition-label>`
 
@@ -681,7 +700,7 @@ def active_cell_property(
 
 @add_method(Case)
 def selected_cell_property_async(
-    self, property_type, property_name, time_step, porosity_model="MATRIX_MODEL"
+    self, property_type, property_name, time_step, porosity_model: str = "MATRIX_MODEL"
 ):
     """Get a cell property for all selected cells. Async, so returns an iterator. For argument details, see :ref:`Result Definition <result-definition-label>`
 
@@ -710,7 +729,7 @@ def selected_cell_property_async(
 
 @add_method(Case)
 def selected_cell_property(
-    self, property_type, property_name, time_step, porosity_model="MATRIX_MODEL"
+    self, property_type, property_name, time_step, porosity_model: str = "MATRIX_MODEL"
 ):
     """Get a cell property for all selected cells. Sync, so returns a list. For argument details, see :ref:`Result Definition <result-definition-label>`
 
@@ -741,7 +760,7 @@ def grid_property_async(
     property_name,
     time_step,
     grid_index=0,
-    porosity_model="MATRIX_MODEL",
+    porosity_model: str = "MATRIX_MODEL",
 ):
     """Get a cell property for all grid cells. Async, so returns an iterator. For argument details, see :ref:`Result Definition <result-definition-label>`
 
@@ -777,7 +796,7 @@ def grid_property(
     property_name,
     time_step,
     grid_index=0,
-    porosity_model="MATRIX_MODEL",
+    porosity_model: str = "MATRIX_MODEL",
 ):
     """Get a cell property for all grid cells. Synchronous, so returns a list. For argument details, see :ref:`Result Definition <result-definition-label>`
 
@@ -808,7 +827,7 @@ def set_active_cell_property_async(
     property_type,
     property_name,
     time_step,
-    porosity_model="MATRIX_MODEL",
+    porosity_model: str = "MATRIX_MODEL",
 ):
     """Set cell property for all active cells Async. Takes an iterator to the input values. For argument details, see :ref:`Result Definition <result-definition-label>`
 
@@ -835,7 +854,12 @@ def set_active_cell_property_async(
 
 @add_method(Case)
 def set_active_cell_property(
-    self, values, property_type, property_name, time_step, porosity_model="MATRIX_MODEL"
+    self,
+    values,
+    property_type,
+    property_name,
+    time_step,
+    porosity_model: str = "MATRIX_MODEL",
 ):
     """Set a cell property for all active cells. For argument details, see :ref:`Result Definition <result-definition-label>`
 
@@ -869,7 +893,7 @@ def set_grid_property(
     property_name,
     time_step,
     grid_index=0,
-    porosity_model="MATRIX_MODEL",
+    porosity_model: str = "MATRIX_MODEL",
 ):
     """Set a cell property for all grid cells. For argument details, see :ref:`Result Definition <result-definition-label>`
 
@@ -984,12 +1008,14 @@ def simulation_wells(self):
         :class:`rips.generated.generated_classes.SimulationWell`
 
     """
-    wells = self.descendants(SimulationWell)
-    return wells
+    if self.views():
+        # Use the first view to get simulation wells.
+        return self.views()[0].descendants(SimulationWell)
+    return []
 
 
 @add_method(Case)
-def active_cell_centers_async(self, porosity_model="MATRIX_MODEL"):
+def active_cell_centers_async(self, porosity_model: str = "MATRIX_MODEL"):
     """Get a cell centers for all active cells. Async, so returns an iterator
 
     Arguments:
@@ -1007,7 +1033,7 @@ def active_cell_centers_async(self, porosity_model="MATRIX_MODEL"):
 
 
 @add_method(Case)
-def active_cell_centers(self, porosity_model="MATRIX_MODEL"):
+def active_cell_centers(self, porosity_model: str = "MATRIX_MODEL"):
     """Get a cell centers for all active cells. Synchronous, so returns a list.
 
     Arguments:
@@ -1025,7 +1051,7 @@ def active_cell_centers(self, porosity_model="MATRIX_MODEL"):
 
 
 @add_method(Case)
-def active_cell_corners_async(self, porosity_model="MATRIX_MODEL"):
+def active_cell_corners_async(self, porosity_model: str = "MATRIX_MODEL"):
     """Get a cell corners for all active cells. Async, so returns an iterator
 
     Arguments:
@@ -1043,7 +1069,7 @@ def active_cell_corners_async(self, porosity_model="MATRIX_MODEL"):
 
 
 @add_method(Case)
-def active_cell_corners(self, porosity_model="MATRIX_MODEL"):
+def active_cell_corners(self, porosity_model: str = "MATRIX_MODEL"):
     """Get a cell corners for all active cells. Synchronous, so returns a list.
 
         Arguments:
@@ -1287,7 +1313,7 @@ def __generate_nnc_property_input_chunks(self, array, parameters):
 
 @add_method(Case)
 def set_nnc_connections_values(
-    self, values, property_name, time_step, porosity_model="MATRIX_MODEL"
+    self, values, property_name, time_step, porosity_model: str = "MATRIX_MODEL"
 ):
     """Set nnc connection values for all connections..
 
@@ -1308,3 +1334,55 @@ def set_nnc_connections_values(
     reply = self.__nnc_properties_stub.SetNNCValues(request_iterator)
     if reply.accepted_value_count < len(values):
         raise IndexError
+
+
+@add_method(EclipseCase)
+def grid_property_for_positions(
+    self,
+    positions: List[List[float]],
+    property_type: str,
+    property_name: str,
+    time_step: int,
+    porosity_model: str = "MATRIX_MODEL",
+) -> List[float]:
+    shared_uuid = uuid.uuid4()
+    coordinate_x = "{}_{}".format(shared_uuid, "coordinate_x")
+    coordinate_y = "{}_{}".format(shared_uuid, "coordinate_y")
+    coordinate_z = "{}_{}".format(shared_uuid, "coordinate_z")
+    result_key = "{}_{}".format(shared_uuid, "result")
+
+    coord_x = []
+    coord_y = []
+    coord_z = []
+    for pos in positions:
+        coord_x.append(pos[0])
+        coord_y.append(pos[1])
+        coord_z.append(pos[2])
+
+        # Get the results from the key-value store.
+    project = self.ancestor(Project)
+    if project:
+        project.set_key_values(coordinate_x, coord_x)
+        project.set_key_values(coordinate_y, coord_y)
+        project.set_key_values(coordinate_z, coord_z)
+
+        self.export_values_internal(
+            coordinate_x=coordinate_x,
+            coordinate_y=coordinate_y,
+            coordinate_z=coordinate_z,
+            result_key=result_key,
+            property_type=property_type,
+            property_name=property_name,
+            time_step=time_step,
+            porosity_model=porosity_model,
+        )
+        result = project.key_values(result_key)
+
+        project.remove_key_values(coordinate_x)
+        project.remove_key_values(coordinate_y)
+        project.remove_key_values(coordinate_z)
+        project.remove_key_values(result_key)
+
+        return result
+    else:
+        return []
