@@ -1,20 +1,21 @@
 """
 ResInsight 3d contour map module
 """
+
 import Commands_pb2
 
 from .pdmobject import add_method
-from .view import View
+from .view import View as View
 from .resinsight_classes import EclipseContourMap, GeoMechContourMap
 
 
 @add_method(EclipseContourMap)
 def export_to_text(
-    self,
-    export_file_name="",
-    export_local_coordinates=False,
-    undefined_value_label="NaN",
-    exclude_undefined_values=False,
+    self: EclipseContourMap,
+    export_file_name: str = "",
+    export_local_coordinates: bool = False,
+    undefined_value_label: str = "NaN",
+    exclude_undefined_values: bool = False,
 ):
     """Export snapshot for the current view
 
@@ -37,11 +38,11 @@ def export_to_text(
 
 @add_method(GeoMechContourMap)
 def export_to_text(
-    self,
-    export_file_name="",
-    export_local_coordinates=False,
-    undefined_value_label="NaN",
-    exclude_undefined_values=False,
+    self: GeoMechContourMap,
+    export_file_name: str = "",
+    export_local_coordinates: bool = False,
+    undefined_value_label: str = "NaN",
+    exclude_undefined_values: bool = False,
 ):
     """Export snapshot for the current view
 
