@@ -3,26 +3,13 @@ Grid Case Group statistics module
 """
 
 from .pdmobject import add_method
-from .view import View
-from .case import Case
+from .view import View as View
+from .case import Case as Case
 
 import Commands_pb2
 from .resinsight_classes import GridCaseGroup
-
-
-@add_method(GridCaseGroup)
-def create_statistics_case(self):
-    """Create a Statistics case in the Grid Case Group
-
-    Returns:
-        :class:`rips.generated.generated_classes.EclipseCase`
-    """
-    command_reply = self._execute_command(
-        createStatisticsCase=Commands_pb2.CreateStatisticsCaseRequest(
-            caseGroupId=self.group_id
-        )
-    )
-    return Case(self.channel, command_reply.createStatisticsCaseResult.caseId)
+from .resinsight_classes import EclipseView
+from .resinsight_classes import RimStatisticalCalculation
 
 
 @add_method(GridCaseGroup)
@@ -33,8 +20,7 @@ def statistics_cases(self):
         List of :class:`rips.generated.generated_classes.EclipseCase`
 
     """
-    stat_case_collection = self.children("StatisticsCaseCollection")[0]
-    return stat_case_collection.children("Reservoirs")
+    return self.descendants(RimStatisticalCalculation)
 
 
 @add_method(GridCaseGroup)
