@@ -3,41 +3,64 @@
 ResInsight caf::PdmObject connection module
 """
 
-from functools import partial, wraps
+from functools import wraps
 import grpc
 import re
-import builtins
-import importlib
 import inspect
-import sys
 
 import PdmObject_pb2
 import PdmObject_pb2_grpc
 import Commands_pb2
 import Commands_pb2_grpc
 
+from .exception import RipsError
 
-def camel_to_snake(name):
+from typing import Any, Callable, TypeVar, Union, List, Optional, Type
+from typing_extensions import ParamSpec
+
+
+def camel_to_snake(name: str) -> str:
     s1 = re.sub("(.)([A-Z][a-z]+)", r"\1_\2", name)
     return re.sub("([a-z0-9])([A-Z])", r"\1_\2", s1).lower()
 
 
-def snake_to_camel(name):
+def snake_to_camel(name: str) -> str:
     return "".join(word.title() for word in name.split("_"))
 
 
-def add_method(cls):
-    def decorator(func):
-        setattr(cls, func.__name__, func)
+F = TypeVar("F", bound=Callable[..., Any])
+C = TypeVar("C")
+
+
+def add_method(cls: C) -> Callable[[F], F]:
+    def decorator(func: F) -> F:
+        def wrapper(*args, **kwargs):
+            try:
+                return func(*args, **kwargs)
+            except grpc.RpcError as e:
+                raise RipsError(e.details()) from None
+
+        # Explicitly preserve signature for Sphinx documentation
+        wrapper.__name__ = func.__name__
+        wrapper.__doc__ = func.__doc__
+        wrapper.__signature__ = inspect.signature(func)  # type: ignore
+        wrapper.__annotations__ = getattr(func, "__annotations__", {})
+
+        # Add the wrapped function to the class
+        setattr(cls, func.__name__, wrapper)
         return func  # returning func means func can still be used normally
 
     return decorator
 
 
-def add_static_method(cls):
-    def decorator(func):
+P = ParamSpec("P")
+T = TypeVar("T")
+
+
+def add_static_method(cls: C) -> Callable[[Callable[P, T]], Callable[P, T]]:
+    def decorator(func: Callable[P, T]) -> Callable[P, T]:
         @wraps(func)
-        def wrapper(*args, **kwargs):
+        def wrapper(*args: P.args, **kwargs: P.kwargs) -> T:
             return func(*args, **kwargs)
 
         setattr(cls, func.__name__, wrapper)
@@ -52,7 +75,9 @@ class PdmObjectBase:
     The ResInsight base class for the Project Data Model
     """
 
-    def _execute_command(self, **command_params):
+    __custom_init__ = None
+
+    def _execute_command(self, **command_params) -> Any:
         self.__warnings = []
         response, call = self._commands.Execute.with_call(
             Commands_pb2.CommandParams(**command_params)
@@ -64,7 +89,11 @@ class PdmObjectBase:
 
         return response
 
-    def __init__(self, pb2_object, channel):
+    def __init__(
+        self,
+        pb2_object: Optional[PdmObject_pb2.PdmObject],
+        channel: Optional[grpc.Channel],
+    ) -> None:
         self.__warnings = []
         self.__chunk_size = 8160
 
@@ -91,11 +120,11 @@ class PdmObjectBase:
             )
             self.__copy_to_pb2()
 
-    def copy_from(self, object):
+    def copy_from(self, obj: object) -> None:
         """Copy attribute values from object to self"""
-        for attribute in dir(object):
+        for attribute in dir(obj):
             if not attribute.startswith("__"):
-                value = getattr(object, attribute)
+                value = getattr(obj, attribute)
                 # This is crucial to avoid overwriting methods
                 if not callable(value):
                     setattr(self, attribute, value)
@@ -103,13 +132,13 @@ class PdmObjectBase:
             self.__custom_init__(self._pb2_object, self._channel)
         self.update()
 
-    def warnings(self):
+    def warnings(self) -> List[str]:
         return self.__warnings
 
-    def has_warnings(self):
+    def has_warnings(self) -> bool:
         return len(self.__warnings) > 0
 
-    def __copy_to_pb2(self):
+    def __copy_to_pb2(self) -> None:
         if self._pb2_object is not None:
             for snake_kw in dir(self):
                 if not snake_kw.startswith("_"):
@@ -119,15 +148,15 @@ class PdmObjectBase:
                         camel_kw = snake_to_camel(snake_kw)
                         self.__set_grpc_value(camel_kw, value)
 
-    def pb2_object(self):
+    def pb2_object(self) -> PdmObject_pb2.PdmObject:
         """Private method"""
         return self._pb2_object
 
-    def channel(self):
+    def channel(self) -> grpc.Channel:
         """Private method"""
         return self._channel
 
-    def address(self):
+    def address(self) -> Any:
         """Get the unique address of the PdmObject
 
         Returns:
@@ -136,21 +165,20 @@ class PdmObjectBase:
 
         return self._pb2_object.address
 
-    def set_visible(self, visible):
+    def set_visible(self, visible: bool) -> None:
         """Set the visibility of the object in the ResInsight project tree"""
         self._pb2_object.visible = visible
 
-    def visible(self):
+    def visible(self) -> bool:
         """Get the visibility of the object in the ResInsight project tree"""
-        return self._pb2_object.visible
+        return bool(self._pb2_object.visible)
 
-    def print_object_info(self):
+    def print_object_info(self) -> None:
         """Print the structure and data content of the PdmObject"""
         print("=========== " + self.__class__.__name__ + " =================")
         print("Object Attributes: ")
         for snake_kw in dir(self):
             if not snake_kw.startswith("_") and not callable(getattr(self, snake_kw)):
-                camel_kw = snake_to_camel(snake_kw)
                 print(
                     "   "
                     + snake_kw
@@ -164,7 +192,10 @@ class PdmObjectBase:
             if not snake_kw.startswith("_") and callable(getattr(self, snake_kw)):
                 print("   " + snake_kw)
 
-    def __convert_from_grpc_value(self, value):
+    Value = Union[bool, str, float, int, "ValueArray"]
+    ValueArray = List[Value]
+
+    def __convert_from_grpc_value(self, value: str) -> Value:
         if value.lower() == "false":
             return False
         if value.lower() == "true":
@@ -183,7 +214,7 @@ class PdmObjectBase:
                     return self.__makelist(value)
                 return value
 
-    def __convert_to_grpc_value(self, value):
+    def __convert_to_grpc_value(self, value: Any) -> str:
         if isinstance(value, bool):
             if value:
                 return "true"
@@ -197,15 +228,15 @@ class PdmObjectBase:
             return "[" + ", ".join(list_of_values) + "]"
         return str(value)
 
-    def __get_grpc_value(self, camel_keyword):
+    def __get_grpc_value(self, camel_keyword: str) -> Value:
         return self.__convert_from_grpc_value(
             self._pb2_object.parameters[camel_keyword]
         )
 
-    def __set_grpc_value(self, camel_keyword, value):
+    def __set_grpc_value(self, camel_keyword: str, value: str) -> None:
         self._pb2_object.parameters[camel_keyword] = self.__convert_to_grpc_value(value)
 
-    def set_value(self, snake_keyword, value):
+    def set_value(self, snake_keyword: str, value: object) -> None:
         """Set the value associated with the provided keyword and updates ResInsight
 
         Arguments:
@@ -217,22 +248,37 @@ class PdmObjectBase:
         setattr(self, snake_keyword, value)
         self.update()
 
-    def __islist(self, value):
+    def __islist(self, value: str) -> bool:
         return value.startswith("[") and value.endswith("]")
 
-    def __makelist(self, list_string):
-        list_string = list_string.lstrip("[")
-        list_string = list_string.rstrip("]")
+    def __makelist(self, list_string: str) -> Value:
+        list_string = list_string.removeprefix("[")
+        list_string = list_string.removesuffix("]")
         if not list_string:
             # Return empty list if empty string. Otherwise, the split function will return ['']
             return []
-        strings = list_string.split(", ")
-        values = []
-        for string in strings:
-            values.append(self.__convert_from_grpc_value(string))
-        return values
 
-    def __from_pb2_to_resinsight_classes(self, pb2_object_list, super_class_definition):
+        # Check if it's a nested list or single list
+        if "], [" in list_string:
+            # Nested list
+            # Split by ], [ to get each sublist
+            sublists = re.split(r"\], \[", list_string)
+            return [self.__makelist(sublist) for sublist in sublists]
+        else:
+            # Single list
+            strings = list_string.split(", ")
+            values = []
+            for string in strings:
+                values.append(self.__convert_from_grpc_value(string))
+            return values
+
+    D = TypeVar("D")
+
+    def __from_pb2_to_resinsight_classes(
+        self,
+        pb2_object_list: List[PdmObject_pb2.PdmObject],
+        super_class_definition: Type[D],
+    ) -> List[D]:
         pdm_object_list = []
         from .generated.generated_classes import class_from_keyword
 
@@ -241,13 +287,15 @@ class PdmObjectBase:
             if child_class_definition is None:
                 child_class_definition = super_class_definition
 
+            assert child_class_definition is not None
             pdm_object = child_class_definition(
                 pb2_object=pb2_object, channel=self.channel()
             )
+
             pdm_object_list.append(pdm_object)
         return pdm_object_list
 
-    def descendants(self, class_definition):
+    def descendants(self, class_definition: Type[D]) -> List[D]:
         """Get a list of all project tree descendants matching the class keyword
         Arguments:
             class_definition[class]: A class definition matching the type of class wanted
@@ -269,7 +317,7 @@ class PdmObjectBase:
                 return []  # Valid empty result
             raise e
 
-    def children(self, child_field, class_definition):
+    def children(self, child_field: str, class_definition: Type[D]) -> List[D]:
         """Get a list of all direct project tree children inside the provided child_field
         Arguments:
             child_field[str]: A field name
@@ -287,7 +335,9 @@ class PdmObjectBase:
                 return []
             raise e
 
-    def add_new_object(self, class_definition, child_field=""):
+    def add_new_object(
+        self, class_definition: Type[D], child_field: str = ""
+    ) -> Optional[D]:
         """Create and add an object to the specified child field
         Arguments:
             class_definition[class]: Class definition of the object to create
@@ -311,18 +361,18 @@ class PdmObjectBase:
             child_class_definition = class_from_keyword(pb2_object.class_keyword)
 
             if child_class_definition is None:
-                child_class_definition = class_keyword
+                child_class_definition = class_definition
+
+            assert child_class_definition.__name__ == class_definition.__name__
+            pdm_object = class_definition(pb2_object=pb2_object, channel=self.channel())
 
-            pdm_object = child_class_definition(
-                pb2_object=pb2_object, channel=self.channel()
-            )
             return pdm_object
         except grpc.RpcError as e:
             if e.code() == grpc.StatusCode.NOT_FOUND:
                 return None
             raise e
 
-    def ancestor(self, class_definition):
+    def ancestor(self, class_definition: Type[D]) -> Optional[D]:
         """Find the first ancestor that matches the provided class_keyword
         Arguments:
             class_definition[class]: A class definition matching the type of class wanted
@@ -330,35 +380,27 @@ class PdmObjectBase:
         assert inspect.isclass(class_definition)
 
         class_keyword = class_definition.__name__
-        from .generated.generated_classes import class_from_keyword
 
         request = PdmObject_pb2.PdmParentObjectRequest(
             object=self._pb2_object, parent_keyword=class_keyword
         )
         try:
             pb2_object = self._pdm_object_stub.GetAncestorPdmObject(request)
-            child_class_definition = class_from_keyword(pb2_object.class_keyword)
-
-            if child_class_definition is None:
-                child_class_definition = class_definition
-
-            pdm_object = child_class_definition(
-                pb2_object=pb2_object, channel=self.channel()
-            )
+            pdm_object = class_definition(pb2_object=pb2_object, channel=self.channel())
             return pdm_object
         except grpc.RpcError as e:
             if e.code() == grpc.StatusCode.NOT_FOUND:
                 return None
             raise e
 
-    def _call_get_method_async(self, method_name):
+    def _call_get_method_async(self, method_name: str):
         request = PdmObject_pb2.PdmObjectGetterRequest(
             object=self._pb2_object, method=method_name
         )
         for chunk in self._pdm_object_stub.CallPdmObjectGetter(request):
             yield chunk
 
-    def _call_get_method(self, method_name):
+    def _call_get_method(self, method_name: str):
         all_values = []
         generator = self._call_get_method_async(method_name)
         for chunk in generator:
@@ -413,7 +455,7 @@ class PdmObjectBase:
         chunk = PdmObject_pb2.PdmObjectSetterChunk()
         yield chunk
 
-    def _call_set_method(self, method_name, values):
+    def _call_set_method(self, method_name: str, values) -> None:
         method_request = PdmObject_pb2.PdmObjectGetterRequest(
             object=self._pb2_object, method=method_name
         )
@@ -422,7 +464,7 @@ class PdmObjectBase:
         if reply.accepted_value_count < len(values):
             raise IndexError
 
-    def _call_pdm_method(self, method_name, **kwargs):
+    def _call_pdm_method_void(self, method_name: str, **kwargs: Any) -> None:
         pb2_params = PdmObject_pb2.PdmObject(class_keyword=method_name)
         for key, value in kwargs.items():
             pb2_params.parameters[snake_to_camel(key)] = self.__convert_to_grpc_value(
@@ -432,20 +474,61 @@ class PdmObjectBase:
             object=self._pb2_object, method=method_name, params=pb2_params
         )
 
-        pb2_object = self._pdm_object_stub.CallPdmObjectMethod(request)
+        try:
+            self._pdm_object_stub.CallPdmObjectMethod(request)
+        except grpc.RpcError as exc:
+            raise RipsError("%s" % exc.details()) from None
 
-        from .generated.generated_classes import class_from_keyword
+    X = TypeVar("X")
 
-        child_class_definition = class_from_keyword(pb2_object.class_keyword)
-        if child_class_definition is None:
-            return None
-
-        pdm_object = child_class_definition(
-            pb2_object=pb2_object, channel=self.channel()
+    def _call_pdm_method_return_value(
+        self, method_name: str, class_definition: Type[X], **kwargs: Any
+    ) -> X:
+        pb2_params = PdmObject_pb2.PdmObject(class_keyword=method_name)
+        for key, value in kwargs.items():
+            pb2_params.parameters[snake_to_camel(key)] = self.__convert_to_grpc_value(
+                value
+            )
+        request = PdmObject_pb2.PdmObjectMethodRequest(
+            object=self._pb2_object, method=method_name, params=pb2_params
         )
-        return pdm_object
 
-    def update(self):
+        try:
+            pb2_object = self._pdm_object_stub.CallPdmObjectMethod(request)
+            pdm_object = class_definition(pb2_object=pb2_object, channel=self.channel())
+            return pdm_object
+        except grpc.RpcError as exc:
+            raise RipsError("%s" % exc.details()) from None
+
+    def _call_pdm_method_return_optional_value(
+        self, method_name: str, class_definition: Type[X], **kwargs: Any
+    ) -> Optional[X]:
+        pb2_params = PdmObject_pb2.PdmObject(class_keyword=method_name)
+        for key, value in kwargs.items():
+            pb2_params.parameters[snake_to_camel(key)] = self.__convert_to_grpc_value(
+                value
+            )
+        request = PdmObject_pb2.PdmObjectMethodRequest(
+            object=self._pb2_object, method=method_name, params=pb2_params
+        )
+
+        try:
+            pb2_object = self._pdm_object_stub.CallPdmObjectMethod(request)
+
+            from .generated.generated_classes import class_from_keyword
+
+            child_class_definition = class_from_keyword(pb2_object.class_keyword)
+            if child_class_definition is None:
+                return None
+
+            assert class_definition.__name__ == child_class_definition.__name__
+            pdm_object = class_definition(pb2_object=pb2_object, channel=self.channel())
+            return pdm_object
+
+        except grpc.RpcError as exc:
+            raise RipsError("%s" % exc.details()) from None
+
+    def update(self) -> None:
         """Sync all fields from the Python Object to ResInsight"""
         self.__copy_to_pb2()
         if self._pdm_object_stub is not None:
@@ -454,3 +537,13 @@ class PdmObjectBase:
             raise Exception(
                 "Object is not connected to GRPC service so cannot update ResInsight"
             )
+
+    def delete(self) -> None:
+        """Delete object in ResInsight"""
+        self.__copy_to_pb2()
+        if self._pdm_object_stub is not None:
+            self._pdm_object_stub.DeleteExistingPdmObject(self._pb2_object)
+        else:
+            raise Exception(
+                "Object is not connected to GRPC service so cannot update ResInsight"
+            )
