@@ -1,10 +1,11 @@
 """
 ResInsight 2d plot module
 """
+
 import Commands_pb2
 
 from .pdmobject import add_method
-from .resinsight_classes import PlotWindow, Plot
+from .resinsight_classes import PlotWindow as PlotWindow, Plot as Plot
 
 
 @add_method(PlotWindow)
