@@ -3,30 +3,40 @@
 """
 The ResInsight project module
 """
-import builtins
+
 import grpc
+import uuid
 
 from .case import Case
 from .gridcasegroup import GridCaseGroup
-from .pdmobject import PdmObjectBase, add_method, add_static_method
-from .plot import Plot
+from .pdmobject import add_method, add_static_method
+from .plot import Plot as Plot
 from .view import View
 
 import Commands_pb2
 from Definitions_pb2 import Empty
 import Project_pb2_grpc
-import Project_pb2
-import PdmObject_pb2
-from .resinsight_classes import Project, PlotWindow, WellPath, SummaryCase
+import KeyValueStore_pb2_grpc
+import KeyValueStore_pb2
+
+from .resinsight_classes import Project, PlotWindow, WellPath, SummaryCase, Reservoir
+
+from typing import Optional, List
 
 
 @add_method(Project)
-def __custom_init__(self, pb2_object, channel):
+def __custom_init__(self, pb2_object, channel: grpc.Channel) -> None:
     self._project_stub = Project_pb2_grpc.ProjectStub(self._channel)
+    self.__key_value_store_stub = KeyValueStore_pb2_grpc.KeyValueStoreStub(
+        self._channel
+    )
+
+    # Public properties
+    self.chunk_size = 8160
 
 
 @add_static_method(Project)
-def create(channel):
+def create(channel: grpc.Channel) -> Project:
     project_stub = Project_pb2_grpc.ProjectStub(channel)
     pb2_object = project_stub.GetPdmObject(Empty())
     return Project(pb2_object, channel)
@@ -56,13 +66,13 @@ def save(self, path=""):
 
 
 @add_method(Project)
-def close(self):
+def close(self) -> None:
     """Close the current project (and open new blank project)"""
     self._execute_command(closeProject=Empty())
 
 
 @add_method(Project)
-def load_case(self, path, grid_only=False):
+def load_case(self: Project, path: str, grid_only: bool = False) -> Reservoir:
     """Load a new grid case from the given file path
 
     Arguments:
@@ -77,7 +87,7 @@ def load_case(self, path, grid_only=False):
 
 
 @add_method(Project)
-def selected_cases(self):
+def selected_cases(self) -> List[Case]:
     """Get a list of all grid cases selected in the project tree
 
     Returns:
@@ -91,17 +101,17 @@ def selected_cases(self):
 
 
 @add_method(Project)
-def cases(self):
+def cases(self: Project) -> List[Reservoir]:
     """Get a list of all grid cases in the project
 
     Returns:
         A list of :class:`rips.generated.generated_classes.Case`
     """
-    return self.descendants(Case)
+    return self.descendants(Reservoir)
 
 
 @add_method(Project)
-def case(self, case_id):
+def case(self: Project, case_id: int) -> Optional[Reservoir]:
     """Get a specific grid case from the provided case Id
 
     Arguments:
@@ -422,3 +432,137 @@ def import_formation_names(self, formation_files=None):
             formationFiles=formation_files, applyToCaseId=-1
         )
     )
+
+
+@add_method(Project)
+def create_corner_point_grid(
+    self,
+    name: str,
+    nx: int,
+    ny: int,
+    nz: int,
+    coord: List[float],
+    zcorn: List[float],
+    actnum: List[int],
+):
+    """Creates a corner point grid from given parameters.
+
+    Arguments:
+        name(str): Name of the grid.
+        nx(int): Number of cells in x direction
+        ny(int): Number of cells in y direction
+        nz(int): Number of cells in z direction
+        coord(list[float]): Coordinate lines as COORD keyword in Eclipse.
+          Each coordinate line is defined by two points (top and bottom).
+          Size: (nx+1) * (ny+1) * 2 * 3. Points are ordered as in Eclipse.
+        zcorn(list[float]): Corner depths as defined by the Eclipse keyword ZCORN.
+          Size: nx * ny * nz * 8
+        actnum(list[int]): Active cell info: cells with values > 0 are active.
+          Size: nx * ny * nz
+    """
+
+    # Generate unique keys for three arrays
+    coord_key = "{}_{}".format(uuid.uuid4(), "coord")
+    zcorn_key = "{}_{}".format(uuid.uuid4(), "zcorn")
+    actnum_key = "{}_{}".format(uuid.uuid4(), "actnum")
+
+    self.set_key_values(coord_key, coord)
+    self.set_key_values(zcorn_key, zcorn)
+    self.set_key_values(actnum_key, actnum)
+    return self.create_grid_from_key_values(
+        name=name,
+        nx=nx,
+        ny=ny,
+        nz=nz,
+        coord_key=coord_key,
+        zcorn_key=zcorn_key,
+        actnum_key=actnum_key,
+    )
+
+
+@add_method(Project)
+def set_key_values(self, key, values):
+    """Sets values for a given key in the key-value store.
+
+    Arguments:
+        key(str): The key (should be unique).
+        values(list): a list of double precision floating point numbers
+    """
+    request_iterator = self.__generate_key_value_store_input_chunks(values, key)
+    reply = self.__key_value_store_stub.SetValue(request_iterator)
+    if reply.accepted_value_count < len(values):
+        raise IndexError
+
+
+@add_method(Project)
+def __generate_key_value_store_input_chunks(self, array, name):
+    index = -1
+    while index < len(array):
+        chunk = KeyValueStore_pb2.KeyValueStoreInputChunk()
+        if index == -1:
+            parameters = KeyValueStore_pb2.KeyValueInputParameters(
+                name=name, num_elements=len(array)
+            )
+            chunk.parameters.CopyFrom(parameters)
+            index += 1
+        else:
+            actual_chunk_size = min(len(array) - index + 1, self.chunk_size)
+            chunk.values.CopyFrom(
+                KeyValueStore_pb2.KeyValueStoreChunk(
+                    values=array[index : index + actual_chunk_size]
+                )
+            )
+            index += actual_chunk_size
+
+        yield chunk
+    # Final empty message to signal completion
+    chunk = KeyValueStore_pb2.KeyValueStoreInputChunk()
+    yield chunk
+
+
+@add_method(Project)
+def key_values_async(
+    self,
+    key,
+):
+    """Get float values for a given key. Async: returns an iterator.
+
+    Arguments:
+        key(str): The key to get.
+
+    Returns:
+        An iterator to a chunk object containing an array of float values
+        Loop through the chunks and then the values within the chunk to get all values.
+    """
+    request = KeyValueStore_pb2.KeyValueStoreOutputRequest(name=key)
+    for chunk in self.__key_value_store_stub.GetValue(request):
+        yield chunk
+
+
+@add_method(Project)
+def key_values(self, key):
+    """Get float values for a given key. Synchronous: returns a list. `
+
+    Arguments:
+        key(str): The key to get.
+
+    Returns:
+        A list of float values
+    """
+    all_values = []
+    generator = self.key_values_async(key)
+    for chunk in generator:
+        for value in chunk.values:
+            all_values.append(value)
+    return all_values
+
+
+@add_method(Project)
+def remove_key_values(self, key):
+    """Remove the values for a given key in the key-value store.
+
+    Arguments:
+        key(str): The key.
+    """
+    request = KeyValueStore_pb2.KeyValueStoreRemoveRequest(name=key)
+    self.__key_value_store_stub.RemoveValue(request)
