@@ -1,6 +1,5 @@
 import sys
 import os
-import grpc
 import pytest
 
 import rips.generated.NNCProperties_pb2 as NNCProperties_pb2
@@ -41,7 +40,7 @@ def test_10kSync(rips_instance, initialize_test):
 
     # Generate some data
     new_data = []
-    for (c, _) in enumerate(nnc_connections):
+    for c, _ in enumerate(nnc_connections):
         new_data.append(float(c))
 
     property_name = "NEW_PROP"
@@ -66,12 +65,12 @@ def test_non_existing_dynamic_values(rips_instance, initialize_test):
     casePath = dataroot.PATH + "/TEST10K_FLT_LGR_NNC/TEST10K_FLT_LGR_NNC.EGRID"
     case = rips_instance.project.load_case(path=casePath)
 
-    with pytest.raises(grpc.RpcError):
+    with pytest.raises(rips.RipsError):
         case.nnc_connections_dynamic_values("x", 0)
 
 
 def test_invalid_time_steps(rips_instance, initialize_test):
     casePath = dataroot.PATH + "/TEST10K_FLT_LGR_NNC/TEST10K_FLT_LGR_NNC.EGRID"
     case = rips_instance.project.load_case(path=casePath)
-    with pytest.raises(grpc.RpcError):
+    with pytest.raises(rips.RipsError):
         case.nnc_connections_generated_values("Formation Allan", 9999)
