@@ -1,7 +1,6 @@
 import sys
 import os
 import pytest
-import grpc
 import tempfile
 
 sys.path.insert(1, os.path.join(sys.path[0], "../../"))
@@ -19,7 +18,7 @@ def test_loadProject(rips_instance, initialize_test):
     assert case.name == "TEST10K_FLT_LGR_NNC"
     assert case.id == 0
     cases = rips_instance.project.cases()
-    assert len(cases) is 1
+    assert len(cases) == 1
 
 
 def test_well_log_plots(rips_instance, initialize_test):
@@ -54,10 +53,6 @@ def test_well_log_plots(rips_instance, initialize_test):
             assert plot2.depth_type == "TRUE_VERTICAL_DEPTH_RKB"
 
 
-@pytest.mark.skipif(
-    sys.platform.startswith("linux"),
-    reason="Brugge is currently exceptionally slow on Linux",
-)
 def test_loadGridCaseGroup(rips_instance, initialize_test):
     case_paths = []
     case_paths.append(dataroot.PATH + "/Case_with_10_timesteps/Real0/BRUGGE_0000.EGRID")
