@@ -1,6 +1,5 @@
 import sys
 import os
-import grpc
 import pytest
 import tempfile
 
@@ -52,7 +51,7 @@ def test_10k_set_out_of_bounds(rips_instance, initialize_test):
 
     results = case.active_cell_property("DYNAMIC_NATIVE", "SOIL", 1)
     results.append(5.0)
-    with pytest.raises(grpc.RpcError):
+    with pytest.raises(rips.RipsError):
         assert case.set_active_cell_property(results, "GENERATED", "SOIL", 1)
 
 
@@ -68,15 +67,15 @@ def test_10k_set_out_of_bounds_client(rips_instance, initialize_test):
 
 
 def createResult(poroChunks, permxChunks):
-    for (poroChunk, permxChunk) in zip(poroChunks, permxChunks):
+    for poroChunk, permxChunk in zip(poroChunks, permxChunks):
         resultChunk = []
-        for (poro, permx) in zip(poroChunk.values, permxChunk.values):
+        for poro, permx in zip(poroChunk.values, permxChunk.values):
             resultChunk.append(poro * permx)
         yield resultChunk
 
 
 def checkResults(poroValues, permxValues, poropermxValues):
-    for (poro, permx, poropermx) in zip(poroValues, permxValues, poropermxValues):
+    for poro, permx, poropermx in zip(poroValues, permxValues, poropermxValues):
         recalc = poro * permx
         assert recalc == pytest.approx(poropermx, rel=1.0e-10)
 
