@@ -2,12 +2,16 @@
 ResInsight 3d view module
 """
 
-import builtins
 import Commands_pb2 as Cmd
 
 import rips.case  # Circular import of Case, which already imports View. Use full name.
 from .pdmobject import add_method
-from .resinsight_classes import View, ViewWindow, EclipseView, GeoMechView
+from .resinsight_classes import (
+    View as View,
+    ViewWindow as ViewWindow,
+    EclipseView as EclipseView,
+    GeoMechView as GeoMechView,
+)
 
 
 @add_method(View)
@@ -187,12 +191,24 @@ def export_property(self, undefined_value=0.0):
     )
 
 
-@add_method(ViewWindow)
+def extract_address(address) -> int:
+    # Address form: "RimReservoir:123345345345435"
+    parts = address.split(":")
+    return int(parts[1])
+
+
+@add_method(View)
 def case(self):
     """Get the case the view belongs to"""
-    mycase = self.ancestor(rips.case.Case)
-    assert mycase is not None
-    return mycase
+    project = self.ancestor(rips.project.Project)
+
+    eclipse_case_addr = extract_address(self.eclipse_case)
+
+    cases = project.cases()
+    for c in cases:
+        if c.address() == eclipse_case_addr:
+            return c
+    return None
 
 
 @add_method(ViewWindow)
